# /usr/bin/env python
from datetime import datetime
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import re
import smtplib
import subprocess
import time


# Run NMAP scan against entire network range
#	-sn: Ping Scan - disable port scan
def run_nmap():
	results = subprocess.check_output(["sudo", "nmap", "-sn", "192.168.0.1/24"])
	return results

# Takes the output from the nmap scan
def parse_nmap():
	# run the scan
	scan = run_nmap()

	#create empty lists to store various data sets
	mac_addresses = []
	ip_addresses = []
	macs_new = []

	# loop through scan results and parse out MAC & IP addresses
	# create lists to hold both
	for line in scan.splitlines():
		mac = re.findall(r"(([A-F\d]{1,2}\:){5}[A-F\d]{1,2})", line)
		mac = [x for x in mac]
		mac_addresses.append(mac)

		macs = [x[0] for x in mac_addresses if x]

	for m in macs:
		macs_new.append(m[0])

	for line in scan.splitlines():

	    ip = re.findall(r'[0-9]+(?:\.[0-9]+){3}', line)
	    if ip:
	    	ip_addresses.append(ip[0])

	# delete the first item from the MAC & IP lists
	# this is the router MAC & IP, not needed
	del ip_addresses[0]
	del macs_new[0]

	# create a dictionary of the two lists
	connected = dict(zip(ip_addresses, macs_new))

	return connected


# Run ARP scan on entire network range
# sends ARP packets to hosts on the local network and displays any responses that are received.
def run_arp():
	results = subprocess.check_output(["arp", "-an"])
	return results

# Parse the results from the arp scan
# Loop through lines and regex out the MAC addresses
def parse_arp():
	# run the scan
	arp = run_arp()

	#create empty lists to store various data sets
	mac_addresses = []
	ip_addresses = []
	macs_new = []

	# loop through scan results and parse out MAC & IP addresses
	# create lists to hold both
	for line in arp.splitlines():
		mac = re.findall(r"(([a-f\d]{1,2}\:){5}[a-f\d]{1,2})", line)
		if mac:
			ip = re.findall(r'[0-9]+(?:\.[0-9]+){3}', line)
			if ip:
				ip_addresses.append(ip[0])

		mac = [x[0] for x in mac]
		mac_addresses.append(mac)
		macs = [x for x in mac_addresses if x]

	for x in macs:
		if x[0].upper() != "FF:FF:FF:FF:FF:FF":
			macs_new.append(x[0].upper())

	# delete un-needed data from lists
	# ie router info, gateway info
	del ip_addresses[:2]
	del ip_addresses[-2:]
	del macs_new[0]
	del macs_new[-1]

	# create a dictionary of the two lists
	connected = dict(zip(ip_addresses, macs_new))
	return connected


# Checks the output from from both scans and compares the outputs
# from both to the list of known MACs
def check_unknown(known_macs, connected):
	#create empty list to hold unknown
	unknown_list = []

	# create string to append to
	# this will be the body of the message
	txt = ""

	# loop through dictionary of IPs/MACs and get the MACs
	for ip in connected:
		mac = re.findall(r"(([a-f\d]{1,2}\:){5}[a-f\d]{1,2})", connected[ip].upper())

		# if the MAC is in known MACs, skip it
		if known_macs.has_key(connected[ip].upper()):
			continue

		# if it is not, it is unknown, append it to the string
		else:
			txt += " Unknown: %s\n" % ip

	return txt

# Send the email/SMS message
def notify(known_macs, unknown):

	# create message body, add new line to skip the Subject field
	message = "\n"

	# append the unknown string from check_unknown() to the message body
	message += unknown

	# gmail email
	gmail_user = 'ztackett11@gmail.com'

	# gmail_password
	gmail_password = 'gmail password goes here'

	# connect to the gmail SMTP server
	server = smtplib.SMTP_SSL('smtp.gmail.com')

	# start the data connection
	server.ehlo()

	# login to gmail
	server.login(gmail_user, gmail_password)

	# send the email/SMS message
	server.sendmail(gmail_user, '7407940674@messaging.sprintpcs.com', message)

	# close the connection to the server
	server.close()


# Get the current time
def get_time():
	localtime = time.asctime(time.localtime(time.time()))

	print str(datetime.now().strftime('\n\nTime Scanned ==> %I:%M: %p'))


def main():

	# get the current time
	get_time()

	# list of known MACs
	known_macs = {
		'30:A1:E5:6D:C6:F0': 'Firestick',
		'8A:8D:1C:0C:23:5B': 'Xbox One',
		'7D:ED:05:17:38:06': 'Huawei Tablet',
		'D0:BA:53:D7:B4:3F': 'Samsung Tablet',
		'F5:1E:15:26:BD:CA': 'Zachs Phone',
		'63:09:D7:3C:D5:80': 'Ambers phone',
		'AF:7F:1A:EE:05:E9': 'Zachs Laptop',
		'69:BD:A7:A5:93:CD': 'Ambers Laptop'
	}

	# get results from the NMAP scan
	nmap = parse_nmap()

	# get results fro thhe ARP scan
	arp = parse_arp()

	# merge the dictionaries from the two different scans
	# both scans can show different results
	# so we want both sets for better accuracy
	connected = arp.copy()
	connected.update(nmap)

	# check to see if there are any unknown devices connected
	unknown = check_unknown(known_macs, connected)

	# if there are unknown devices, send email/SMS with notification
	if unknown:
		notify(known_macs, unknown)

if __name__ == "__main__":
	main()
